@@ -8,7 +8,6 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
-	"log"
 	"net"
 	"net/http"
 	"net/http/pprof"
@@ -18,26 +17,36 @@ import (
 	"os/user"
 	"path"
 	"path/filepath"
+	"strconv"
 	"strings"
+	"sync"
 	"time"
 
+	"github.com/anacrolix/log"
+
 	"github.com/anacrolix/dms/dlna"
 	"github.com/anacrolix/dms/soap"
 	"github.com/anacrolix/dms/ssdp"
 	"github.com/anacrolix/dms/transcode"
 	"github.com/anacrolix/dms/upnp"
 	"github.com/anacrolix/dms/upnpav"
+	"github.com/anacrolix/dms/version"
 	"github.com/anacrolix/ffprobe"
 )
 
+var (
+	serverField = fmt.Sprintf(`Linux/3.4 DLNADOC/1.50 UPnP/1.0 %s/%s`,
+		userAgentProduct,
+		version.DmsVersion)
+	rootDeviceModelName = fmt.Sprintf("%s %s", userAgentProduct, version.DmsVersion)
+)
+
 const (
-	serverField                 = "Linux/3.4 DLNADOC/1.50 UPnP/1.0 DMS/1.0"
+	userAgentProduct            = "dms"
 	rootDeviceType              = "urn:schemas-upnp-org:device:MediaServer:1"
-	rootDeviceModelName         = "dms 1.0"
 	resPath                     = "/res"
 	iconPath                    = "/icon"
 	rootDescPath                = "/rootDesc.xml"
-	contentDirectorySCPDURL     = "/scpd/ContentDirectory.xml"
 	contentDirectoryEventSubURL = "/evt/ContentDirectory"
 	serviceControlURL           = "/ctl"
 	deviceIconPath              = "/deviceIcon"
@@ -57,6 +66,7 @@ var transcodes = map[string]transcodeSpec{
 	},
 	"vp8":        {mimeType: "video/webm", Transcode: transcode.VP8Transcode},
 	"chromecast": {mimeType: "video/mp4", Transcode: transcode.ChromecastTranscode},
+	"web":        {mimeType: "video/mp4", Transcode: transcode.WebTranscode},
 }
 
 func makeDeviceUuid(unique string) string {
@@ -84,13 +94,20 @@ var services = []*service{
 		},
 		SCPD: contentDirectoryServiceDescription,
 	},
-	// {
-	// 	Service: upnp.Service{
-	// 		ServiceType: "urn:schemas-upnp-org:service:ConnectionManager:3",
-	// 		ServiceId:   "urn:upnp-org:serviceId:ConnectionManager",
-	// 	},
-	// 	SCPD: connectionManagerServiceDesc,
-	// },
+	{
+		Service: upnp.Service{
+			ServiceType: "urn:schemas-upnp-org:service:ConnectionManager:1",
+			ServiceId:   "urn:upnp-org:serviceId:ConnectionManager",
+		},
+		SCPD: connectionManagerServiceDescription,
+	},
+	{
+		Service: upnp.Service{
+			ServiceType: "urn:microsoft.com:service:X_MS_MediaReceiverRegistrar:1",
+			ServiceId:   "urn:microsoft.com:serviceId:X_MS_MediaReceiverRegistrar",
+		},
+		SCPD: mediaReceiverRegistrarDescription,
+	},
 }
 
 // The control URL for every service is the same. We're able to infer the desired service from the request headers.
@@ -112,6 +129,7 @@ func serviceTypes() (ret []string) {
 	}
 	return
 }
+
 func (me *Server) httpPort() int {
 	return me.HTTPConn.Addr().(*net.TCPAddr).Port
 }
@@ -145,25 +163,21 @@ func (me *Server) serveHTTP() error {
 const ssdpInterfaceFlags = net.FlagUp | net.FlagMulticast
 
 func (me *Server) doSSDP() {
-	active := 0
-	stopped := make(chan struct{})
+	var wg sync.WaitGroup
 	for _, if_ := range me.Interfaces {
-		active++
-		go func(if_ net.Interface) {
-			defer func() {
-				stopped <- struct{}{}
-			}()
+		if_ := if_
+		wg.Add(1)
+		go func() {
+			defer wg.Done()
 			me.ssdpInterface(if_)
-		}(if_)
-	}
-	for active > 0 {
-		<-stopped
-		active--
+		}()
 	}
+	wg.Wait()
 }
 
 // Run SSDP server on an interface.
 func (me *Server) ssdpInterface(if_ net.Interface) {
+	logger := me.Logger.WithNames("ssdp", if_.Name)
 	s := ssdp.Server{
 		Interface: if_,
 		Devices:   devices(),
@@ -174,6 +188,7 @@ func (me *Server) ssdpInterface(if_ net.Interface) {
 		Server:         serverField,
 		UUID:           me.rootDeviceUUID,
 		NotifyInterval: me.NotifyInterval,
+		Logger:         logger,
 	}
 	if err := s.Init(); err != nil {
 		if if_.Flags&ssdpInterfaceFlags != ssdpInterfaceFlags {
@@ -186,16 +201,16 @@ func (me *Server) ssdpInterface(if_ net.Interface) {
 			// good.
 			return
 		}
-		log.Printf("error creating ssdp server on %s: %s", if_.Name, err)
+		logger.Printf("error creating ssdp server on %s: %s", if_.Name, err)
 		return
 	}
 	defer s.Close()
-	log.Println("started SSDP on", if_.Name)
+	logger.Levelf(log.Info, "started SSDP on %q", if_.Name)
 	stopped := make(chan struct{})
 	go func() {
 		defer close(stopped)
 		if err := s.Serve(); err != nil {
-			log.Printf("%q: %q\n", if_.Name, err)
+			logger.Printf("%q: %q\n", if_.Name, err)
 		}
 	}()
 	select {
@@ -205,27 +220,27 @@ func (me *Server) ssdpInterface(if_ net.Interface) {
 	}
 }
 
-var (
-	startTime time.Time
-)
+var startTime time.Time
 
 type Icon struct {
 	Width, Height, Depth int
 	Mimetype             string
-	io.ReadSeeker
+	Bytes                []byte
 }
 
 type Server struct {
-	HTTPConn       net.Listener
-	FriendlyName   string
-	Interfaces     []net.Interface
-	httpServeMux   *http.ServeMux
-	RootObjectPath string
-	rootDescXML    []byte
-	rootDeviceUUID string
-	FFProbeCache   Cache
-	closed         chan struct{}
-	ssdpStopped    chan struct{}
+	HTTPConn               net.Listener
+	FriendlyName           string
+	Interfaces             []net.Interface
+	httpServeMux           *http.ServeMux
+	RootObjectPath         string
+	OnBrowseDirectChildren func(path string, rootObjectPath string, host, userAgent string) (ret []interface{}, err error)
+	OnBrowseMetadata       func(path string, rootObjectPath string, host, userAgent string) (ret interface{}, err error)
+	rootDescXML            []byte
+	rootDeviceUUID         string
+	FFProbeCache           Cache
+	closed                 chan struct{}
+	ssdpStopped            chan struct{}
 	// The service SOAP handler keyed by service URN.
 	services   map[string]UPnPService
 	LogHeaders bool
@@ -246,12 +261,14 @@ type Server struct {
 	// Ingnore unreadable files and directories
 	IgnoreUnreadable bool
 	// White list of clients
-	AllowedIpNets []*net.IPNet
+	AllowedIpNets  []*net.IPNet
+	Logger         log.Logger
+	eventingLogger log.Logger
 }
 
 // UPnP SOAP service.
 type UPnPService interface {
-	Handle(action string, argsXML []byte, r *http.Request) (respArgs map[string]string, err error)
+	Handle(action string, argsXML []byte, r *http.Request) (respArgs [][2]string, err error)
 	Subscribe(callback []*url.URL, timeoutSeconds int) (sid string, actualTimeout int, err error)
 	Unsubscribe(sid string) error
 }
@@ -393,7 +410,7 @@ func (me *Server) serveDLNATranscode(w http.ResponseWriter, r *http.Request, pat
 		u, _ := user.Current()
 		return filepath.Join(u.HomeDir, ".dms", "log", tsname, filepath.Base(path_))
 	}()
-	os.MkdirAll(filepath.Dir(stderrPath), 0750)
+	os.MkdirAll(filepath.Dir(stderrPath), 0o750)
 	logFile, err := os.Create(stderrPath)
 	if err != nil {
 		log.Printf("couldn't create transcode log file: %s", err)
@@ -426,19 +443,22 @@ func init() {
 }
 
 func getDefaultFriendlyName() string {
-	return fmt.Sprintf("%s: %s on %s", rootDeviceModelName, func() string {
-		user, err := user.Current()
-		if err != nil {
-			log.Panicf("getDefaultFriendlyName could not get username: %s", err)
-		}
-		return user.Name
-	}(), func() string {
-		name, err := os.Hostname()
-		if err != nil {
-			log.Panicf("getDefaultFriendlyName could not get hostname: %s", err)
-		}
-		return name
-	}())
+	return fmt.Sprintf("%s: %s on %s",
+		rootDeviceModelName,
+		func() string {
+			user, err := user.Current()
+			if err != nil {
+				log.Panicf("getDefaultFriendlyName could not get username: %s", err)
+			}
+			return user.Name
+		}(),
+		func() string {
+			name, err := os.Hostname()
+			if err != nil {
+				log.Panicf("getDefaultFriendlyName could not get hostname: %s", err)
+			}
+			return name
+		}())
 }
 
 func xmlMarshalOrPanic(value interface{}) []byte {
@@ -487,8 +507,9 @@ func (me *mitmRespWriter) CloseNotify() <-chan bool {
 // Set the SCPD serve paths.
 func init() {
 	for _, s := range services {
-		p := path.Join("/scpd", s.ServiceId)
-		s.SCPDURL = p
+		lastInd := strings.LastIndex(s.ServiceId, ":")
+		p := path.Join("/scpd", s.ServiceId[lastInd+1:])
+		s.SCPDURL = p + ".xml"
 	}
 }
 
@@ -498,16 +519,17 @@ func handleSCPDs(mux *http.ServeMux) {
 		mux.HandleFunc(s.SCPDURL, func(serviceDesc string) http.HandlerFunc {
 			return func(w http.ResponseWriter, r *http.Request) {
 				w.Header().Set("content-type", `text/xml; charset="utf-8"`)
-				http.ServeContent(w, r, ".xml", startTime, bytes.NewReader([]byte(serviceDesc)))
+				http.ServeContent(w, r, "", startTime, bytes.NewReader([]byte(serviceDesc)))
 			}
 		}(s.SCPD))
 	}
 }
 
 // Marshal SOAP response arguments into a response XML snippet.
-func marshalSOAPResponse(sa upnp.SoapAction, args map[string]string) []byte {
+func marshalSOAPResponse(sa upnp.SoapAction, args [][2]string) []byte {
 	soapArgs := make([]soap.Arg, 0, len(args))
-	for argName, value := range args {
+	for _, arg := range args {
+		argName, value := arg[0], arg[1]
 		soapArgs = append(soapArgs, soap.Arg{
 			XMLName: xml.Name{Local: argName},
 			Value:   value,
@@ -517,7 +539,7 @@ func marshalSOAPResponse(sa upnp.SoapAction, args map[string]string) []byte {
 }
 
 // Handle a SOAP request and return the response arguments or UPnP error.
-func (me *Server) soapActionResponse(sa upnp.SoapAction, actionRequestXML []byte, r *http.Request) (map[string]string, error) {
+func (me *Server) soapActionResponse(sa upnp.SoapAction, actionRequestXML []byte, r *http.Request) ([][2]string, error) {
 	service, ok := me.services[sa.Type]
 	if !ok {
 		// TODO: What's the invalid service error?!
@@ -530,6 +552,10 @@ func (me *Server) soapActionResponse(sa upnp.SoapAction, actionRequestXML []byte
 func (me *Server) serviceControlHandler(w http.ResponseWriter, r *http.Request) {
 	found := false
 	clientIp, _, _ := net.SplitHostPort(r.RemoteAddr)
+	if zoneDelimiterIdx := strings.Index(clientIp, "%"); zoneDelimiterIdx != -1 {
+		// IPv6 addresses may have the form address%zone (e.g. ::1%eth0)
+		clientIp = clientIp[:zoneDelimiterIdx]
+	}
 	for _, ipnet := range me.AllowedIpNets {
 		if ipnet.Contains(net.ParseIP(clientIp)) {
 			found = true
@@ -551,8 +577,8 @@ func (me *Server) serviceControlHandler(w http.ResponseWriter, r *http.Request)
 		http.Error(w, err.Error(), http.StatusBadRequest)
 		return
 	}
-	//AwoX/1.1 UPnP/1.0 DLNADOC/1.50
-	//log.Println(r.UserAgent())
+	// AwoX/1.1 UPnP/1.0 DLNADOC/1.50
+	// log.Println(r.UserAgent())
 	w.Header().Set("Content-Type", `text/xml; charset="utf-8"`)
 	w.Header().Set("Ext", "")
 	w.Header().Set("Server", serverField)
@@ -589,7 +615,10 @@ func (me *Server) serveIcon(w http.ResponseWriter, r *http.Request) {
 	// cmd.Stderr = os.Stderr
 	body, err := cmd.Output()
 	if err != nil {
-		http.Error(w, err.Error(), http.StatusInternalServerError)
+		// serve 1st Icon if no ffmpegthumbnailer
+		w.Header().Set("Content-Type", me.Icons[0].Mimetype)
+		http.ServeContent(w, r, "", time.Time{}, bytes.NewReader(me.Icons[0].Bytes))
+		// http.Error(w, err.Error(), http.StatusInternalServerError)
 		return
 	}
 	http.ServeContent(w, r, "", time.Now(), bytes.NewReader(body))
@@ -598,7 +627,7 @@ func (me *Server) serveIcon(w http.ResponseWriter, r *http.Request) {
 func (server *Server) contentDirectoryInitialEvent(urls []*url.URL, sid string) {
 	body := xmlMarshalOrPanic(upnp.PropertySet{
 		Properties: []upnp.Property{
-			upnp.Property{
+			{
 				Variable: upnp.Variable{
 					XMLName: xml.Name{
 						Local: "SystemUpdateID",
@@ -624,7 +653,7 @@ func (server *Server) contentDirectoryInitialEvent(urls []*url.URL, sid string)
 		Space: "urn:schemas-upnp-org:event-1-0",
 	})
 	body = append([]byte(`<?xml version="1.0"?>`+"\n"), body...)
-	eventingLogger.Print(string(body))
+	server.eventingLogger.Print(string(body))
 	for _, _url := range urls {
 		bodyReader := bytes.NewReader(body)
 		req, err := http.NewRequest("NOTIFY", _url.String(), bodyReader)
@@ -639,23 +668,21 @@ func (server *Server) contentDirectoryInitialEvent(urls []*url.URL, sid string)
 		req.Header["SEQ"] = []string{"0"}
 		// req.Header["TRANSFER-ENCODING"] = []string{"chunked"}
 		// req.ContentLength = int64(bodyReader.Len())
-		eventingLogger.Print(req.Header)
-		eventingLogger.Print("starting notify")
+		server.eventingLogger.Print(req.Header)
+		server.eventingLogger.Print("starting notify")
 		resp, err := http.DefaultClient.Do(req)
-		eventingLogger.Print("finished notify")
+		server.eventingLogger.Print("finished notify")
 		if err != nil {
 			log.Printf("Could not notify %s: %s", _url.String(), err)
 			continue
 		}
-		eventingLogger.Print(resp)
+		server.eventingLogger.Print(resp)
 		b, _ := ioutil.ReadAll(resp.Body)
-		eventingLogger.Println(string(b))
+		server.eventingLogger.Println(string(b))
 		resp.Body.Close()
 	}
 }
 
-var eventingLogger = log.New(ioutil.Discard, "", 0)
-
 func (server *Server) contentDirectoryEventSubHandler(w http.ResponseWriter, r *http.Request) {
 	if server.StallEventSubscribe {
 		// I have an LG TV that doesn't like my eventing implementation.
@@ -674,21 +701,21 @@ func (server *Server) contentDirectoryEventSubHandler(w http.ResponseWriter, r *
 		// TODO: Get eventing to work with the problematic TV.
 		t := time.Now()
 		<-w.(http.CloseNotifier).CloseNotify()
-		eventingLogger.Printf("stalled subscribe connection went away after %s", time.Since(t))
+		server.eventingLogger.Printf("stalled subscribe connection went away after %s", time.Since(t))
 		return
 	}
 	// The following code is a work in progress. It partially implements
 	// the spec on eventing but hasn't been completed as I have nothing to
 	// test it with.
-	eventingLogger.Print(r.Header)
+	server.eventingLogger.Print(r.Header)
 	service := server.services["ContentDirectory"]
-	eventingLogger.Println(r.RemoteAddr, r.Method, r.Header.Get("SID"))
+	server.eventingLogger.Println(r.RemoteAddr, r.Method, r.Header.Get("SID"))
 	if r.Method == "SUBSCRIBE" && r.Header.Get("SID") == "" {
 		urls := upnp.ParseCallbackURLs(r.Header.Get("CALLBACK"))
-		eventingLogger.Println(urls)
+		server.eventingLogger.Println(urls)
 		var timeout int
 		fmt.Sscanf(r.Header.Get("TIMEOUT"), "Second-%d", &timeout)
-		eventingLogger.Println(timeout, r.Header.Get("TIMEOUT"))
+		server.eventingLogger.Println(timeout, r.Header.Get("TIMEOUT"))
 		sid, timeout, _ := service.Subscribe(urls, timeout)
 		w.Header()["SID"] = []string{sid}
 		w.Header()["TIMEOUT"] = []string{fmt.Sprintf("Second-%d", timeout)}
@@ -701,11 +728,12 @@ func (server *Server) contentDirectoryEventSubHandler(w http.ResponseWriter, r *
 	} else if r.Method == "SUBSCRIBE" {
 		http.Error(w, "meh", http.StatusPreconditionFailed)
 	} else {
-		eventingLogger.Printf("unhandled event method: %s", r.Method)
+		server.eventingLogger.Printf("unhandled event method: %s", r.Method)
 	}
 }
 
 func (server *Server) initMux(mux *http.ServeMux) {
+	// Handle root (presentationURL)
 	mux.HandleFunc("/", func(resp http.ResponseWriter, req *http.Request) {
 		resp.Header().Set("content-type", "text/html")
 		err := rootTmpl.Execute(resp, struct {
@@ -736,8 +764,8 @@ func (server *Server) initMux(mux *http.ServeMux) {
 		} else {
 			k = r.URL.Query().Get("transcode")
 		}
-		if k == "" {
-			mimeType, err := MimeTypeByPath(filePath)
+		mimeType, err := MimeTypeByPath(filePath)
+		if k == "" || mimeType.IsImage() {
 			if err != nil {
 				http.Error(w, err.Error(), http.StatusInternalServerError)
 				return
@@ -766,11 +794,19 @@ func (server *Server) initMux(mux *http.ServeMux) {
 	handleSCPDs(mux)
 	mux.HandleFunc(serviceControlURL, server.serviceControlHandler)
 	mux.HandleFunc("/debug/pprof/", pprof.Index)
-	for i, di := range server.Icons {
-		mux.HandleFunc(fmt.Sprintf("%s/%d", deviceIconPath, i), func(w http.ResponseWriter, r *http.Request) {
-			w.Header().Set("Content-Type", di.Mimetype)
-			http.ServeContent(w, r, "", time.Time{}, di.ReadSeeker)
-		})
+	// DeviceIcons
+	iconHandl := func(w http.ResponseWriter, r *http.Request) {
+		idStr := path.Base(r.URL.Path)
+		id, _ := strconv.Atoi(idStr)
+		if id < 0 || id >= len(server.Icons) {
+			id = 0
+		}
+		di := server.Icons[id]
+		w.Header().Set("Content-Type", di.Mimetype)
+		http.ServeContent(w, r, "", time.Time{}, bytes.NewReader(di.Bytes))
+	}
+	for i := range server.Icons {
+		mux.HandleFunc(fmt.Sprintf("%s/%d", deviceIconPath, i), iconHandl)
 	}
 }
 
@@ -779,15 +815,31 @@ func (s *Server) initServices() (err error) {
 	if err != nil {
 		return
 	}
+	urn1, err := upnp.ParseServiceType(services[1].ServiceType)
+	if err != nil {
+		return
+	}
+	urn2, err := upnp.ParseServiceType(services[2].ServiceType)
+	if err != nil {
+		return
+	}
 	s.services = map[string]UPnPService{
 		urn.Type: &contentDirectoryService{
 			Server: s,
 		},
+		urn1.Type: &connectionManagerService{
+			Server: s,
+		},
+		urn2.Type: &mediaReceiverRegistrarService{
+			Server: s,
+		},
 	}
 	return
 }
 
 func (srv *Server) Init() (err error) {
+	srv.eventingLogger = srv.Logger.WithNames("eventing")
+	srv.eventingLogger.Levelf(log.Debug, "hello %v", "world")
 	if err = srv.initServices(); err != nil {
 		return
 	}
@@ -822,6 +874,8 @@ func (srv *Server) Init() (err error) {
 	srv.rootDeviceUUID = makeDeviceUuid(srv.FriendlyName)
 	srv.rootDescXML, err = xml.MarshalIndent(
 		upnp.DeviceDesc{
+			NSDLNA:      "urn:schemas-dlna-org:device-1-0",
+			NSSEC:       "http://www.sec.co.kr/dlna",
 			SpecVersion: upnp.SpecVersion{Major: 1, Minor: 0},
 			Device: upnp.Device{
 				DeviceType:   rootDeviceType,
@@ -829,6 +883,12 @@ func (srv *Server) Init() (err error) {
 				Manufacturer: "Matt Joiner <anacrolix@gmail.com>",
 				ModelName:    rootDeviceModelName,
 				UDN:          srv.rootDeviceUUID,
+				VendorXML: `
+     <dlna:X_DLNACAP/>
+     <dlna:X_DLNADOC>DMS-1.50</dlna:X_DLNADOC>
+     <dlna:X_DLNADOC>M-DMS-1.50</dlna:X_DLNADOC>
+     <sec:ProductCap>smi,DCM10,getMediaInfo.sec,getCaptionInfo.sec</sec:ProductCap>
+     <sec:X_ProductCap>smi,DCM10,getMediaInfo.sec,getCaptionInfo.sec</sec:X_ProductCap>`,
 				ServiceList: func() (ss []upnp.Service) {
 					for _, s := range services {
 						ss = append(ss, s.Service)
@@ -847,6 +907,7 @@ func (srv *Server) Init() (err error) {
 					}
 					return
 				}(),
+				PresentationURL: "/",
 			},
 		},
 		" ", "  ")
@@ -854,7 +915,7 @@ func (srv *Server) Init() (err error) {
 		return
 	}
 	srv.rootDescXML = append([]byte(`<?xml version="1.0"?>`), srv.rootDescXML...)
-	log.Println("HTTP srv on", srv.HTTPConn.Addr())
+	srv.Logger.Println("HTTP srv on", srv.HTTPConn.Addr())
 	srv.initMux(srv.httpServeMux)
 	srv.ssdpStopped = make(chan struct{})
 	return nil
