@@ -1,11 +1,13 @@
+//go:build ignore
 // +build ignore
 
 package main
 
 import (
 	"fmt"
-	"log"
 	"net/url"
+
+	"github.com/anacrolix/log"
 )
 
 func main() {
