@@ -4,12 +4,13 @@ package transcode
 
 import (
 	"io"
-	"log"
 	"os/exec"
 	"runtime"
 	"strconv"
 	"time"
 
+	"github.com/anacrolix/log"
+
 	. "github.com/anacrolix/dms/misc"
 	"github.com/anacrolix/ffprobe"
 )
@@ -114,7 +115,8 @@ func VP8Transcode(path string, start, length time.Duration, stderr io.Writer) (r
 		// "-deadline", "good",
 		// "-c:v", "libvpx", "-crf", "10",
 		"-f", "webm",
-		"pipe:"}...)
+		"pipe:",
+	}...)
 	return transcodePipe(args, stderr)
 }
 
@@ -134,6 +136,31 @@ func ChromecastTranscode(path string, start, length time.Duration, stderr io.Wri
 	}
 	args = append(args, []string{
 		"-f", "mp4",
-		"pipe:"}...)
+		"pipe:",
+	}...)
+	return transcodePipe(args, stderr)
+}
+
+// Returns a stream of h264 video and mp3 audio
+func WebTranscode(path string, start, length time.Duration, stderr io.Writer) (r io.ReadCloser, err error) {
+	args := []string{
+		"ffmpeg",
+		"-ss", FormatDurationSexagesimal(start),
+		"-i", path,
+		"-pix_fmt", "yuv420p",
+		"-c:v", "libx264", "-crf", "25",
+		"-c:a", "mp3", "-ab", "128k", "-ar", "44100",
+		"-preset", "ultrafast",
+		"-movflags", "+faststart+frag_keyframe+empty_moov",
+	}
+	if length > 0 {
+		args = append(args, []string{
+			"-t", FormatDurationSexagesimal(length),
+		}...)
+	}
+	args = append(args, []string{
+		"-f", "mp4",
+		"pipe:",
+	}...)
 	return transcodePipe(args, stderr)
 }
